// Package middleware provides graphql middlewares for resolvers
package middleware

import (
	"fmt"
	"runtime/debug"

	"github.com/graphql-go/graphql"
	"gitlab.mashroom.com/mashroom/backend/common/api/grpcerr"
)

// GraphqlMiddleware wraps field resolver with panics interceptor and errors detector.
func GraphqlMiddleware(f graphql.FieldResolveFn) graphql.FieldResolveFn {
	return func(p graphql.ResolveParams) (res interface{}, err error) {
		defer func() {
			r := recover()
			if r != nil {
				var codeerr *grpcerr.Error

				if err == nil {
					codeerr = &grpcerr.Error{
						Message:  fmt.Sprint(r),
						Code:     "unknown",
						Service:  "",
						Original: nil,
					}
				} else {
					codeerr = grpcerr.Wrap(err)
				}

				codeerr.Stack = string(debug.Stack())

				err = codeerr
			} else if err != nil {
				err = grpcerr.Wrap(err)
			}
		}()

		return f(p)
	}
}

// GraphqlMiddlewareObjectFields applies middleware.GraphqlMiddleware to fields resolvers.
func GraphqlMiddlewareObjectFields(fields graphql.Fields) graphql.Fields {
	for _, f := range fields {
		f.Resolve = GraphqlMiddleware(f.Resolve)
	}

	return fields
}

// GraphqlMiddlewareObject applies middleware.GraphqlMiddlewareObjectFields to object fields.
func GraphqlMiddlewareObject(o graphql.ObjectConfig) graphql.ObjectConfig {
	switch fields := o.Fields.(type) {
	case graphql.Fields:
		o.Fields = GraphqlMiddlewareObjectFields(fields)
	case graphql.FieldsThunk:
		o.Fields = func() graphql.Fields {
			return GraphqlMiddlewareObjectFields(fields())
		}
	default:
		panic("unknown fields type")
	}

	return o
}
