package admin

import (
	"github.com/graphql-go/graphql"
	"gitlab.mashroom.com/mashroom/backend/common/graphql/util"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/connections"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/resolvers"
	invoice "gitlab.mashroom.com/mashroom/backend/invoice/api"
)

func initOrder(query, mutation *graphql.Fields) {
	(*query)["products"] = &graphql.Field{
		Description: "List all property products",
		Args:        util.BindArguments(invoice.ProductListRequest{}),
		Resolve:     adminQueryProductsResolver,
		Type:        util.BindOutput(invoice.ProductListResponse{}, "Product list", nil),
	}
}

func adminQueryProductsResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &invoice.ProductListRequest{}

	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	return connections.InvoiceService.ProductList(params.Context, request)
}
