package resolvers

import (
	"github.com/graphql-go/graphql"

	"gitlab.mashroom.com/mashroom/backend/common/graphql/util"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/connections"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/middleware"
	grabdata "gitlab.mashroom.com/mashroom/backend/grabdata/api"
)

func initGrabdata(rootQueries, rootMutations *graphql.Fields) {
	query := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "GrabdataQuery",
		Description: "Grabdata query root",
		Fields: graphql.Fields{
			"lettingagents": &graphql.Field{
				Description: "Returns letting agents list",
				Args:        util.BindArguments(grabdata.ListLettingAgentsRequest{}),
				Resolve:     GrabdataLettingAgentsListResolver,
				Type:        util.BindOutput(grabdata.ListLettingAgentsResponse{}, "Letting agents list response", nil),
			},
			"areainfo": &graphql.Field{
				Description: "Returns area info",
				Args:        util.BindArguments(grabdata.GetAggregatedAreaInfoRequest{}),
				Resolve:     GrabdataAreaInfoResolver,
				Type:        util.BindOutput(grabdata.AreaInfoResponse{}, "Area info response", nil),
			},
			"getagentscounters": &graphql.Field{
				Description: "Returns agents counters list by locations",
				Args:        util.BindArguments(grabdata.GetAgentsCountByLocationsRequest{}),
				Resolve:     GrabdataAgentsCountersListResolver,
				Type: util.BindOutput(grabdata.GetAgentsCountByLocationsResponse{},
					"Agents counters list response", nil),
			},
		},
	}))

	(*rootQueries)["grabdata"] = &graphql.Field{
		Type: query,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return query, nil
		},
	}
}

func GrabdataLettingAgentsListResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &grabdata.ListLettingAgentsRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	return connections.GrabDataService.ListLettingAgents(params.Context, request)
}

func GrabdataAreaInfoResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &grabdata.GetAggregatedAreaInfoRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	return connections.GrabDataService.GetAggregatedAreaInfo(params.Context, request)
}

func GrabdataAgentsCountersListResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &grabdata.GetAgentsCountByLocationsRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	return connections.GrabDataService.GetAgentsCount(params.Context, request)
}
