package resolvers

import (
	"github.com/graphql-go/graphql"
	auth "gitlab.mashroom.com/mashroom/backend/auth/api"
	"gitlab.mashroom.com/mashroom/backend/common/graphql/util"
	ws "gitlab.mashroom.com/mashroom/backend/common/graphql/websocket"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/connections"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/middleware"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/resolvers/errs"
	invoice "gitlab.mashroom.com/mashroom/backend/invoice/api"
)

func initInvoice(rootQueries, rootMutations *graphql.Fields) {
	query := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "InvoiceQuery",
		Description: "invoice query root",
		Fields: graphql.Fields{
			"list": &graphql.Field{
				Description: "Get own invoices",
				Resolve:     InvoiceListQueryResolver,
				Args:        util.BindArguments(invoice.InvoiceListRequest{}),
				Type:        util.BindOutput(invoice.InvoiceListResponse{}, "Invoice list", nil),
			},
		},
	}))

	(*rootQueries)["invoice"] = &graphql.Field{
		Type: query,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return query, nil
		},
	}
}

func InvoiceListQueryResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}

	request := &invoice.InvoiceListRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	request.User = user.GetAccount().GetIdValue()

	return connections.InvoiceService.InvoiceList(params.Context, request)
}
