package resolvers

import (
	"fmt"

	"github.com/graphql-go/graphql"
	auth "gitlab.mashroom.com/mashroom/backend/auth/api"
	"gitlab.mashroom.com/mashroom/backend/common/graphql/util"
	ws "gitlab.mashroom.com/mashroom/backend/common/graphql/websocket"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/connections"
	"gitlab.mashroom.com/mashroom/backend/gateway/impl/middleware"
	notification "gitlab.mashroom.com/mashroom/backend/notification/api"
)

func initNotification(rootQueries, rootMutations *graphql.Fields) {
	subscription := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "NotificationSubscriptionMutation",
		Description: "Notification subscription mutation root",
		Fields: graphql.Fields{
			"leads": &graphql.Field{
				Description: "Subscribe to leads list",
				Args:        util.BindArguments(notification.ListSubscribeLeads{}),
				Resolve:     NotificationSubscriptionMutationLeadsResolver,
				Type:        graphql.Boolean,
			},
		},
	}))

	mutation := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "NotificationMutation",
		Description: "Notification mutation root",
		Fields: graphql.Fields{
			"subscribe": &graphql.Field{
				Resolve: func(p graphql.ResolveParams) (interface{}, error) {
					return subscription, nil
				},
				Type: subscription,
			},
		},
	}))

	(*rootMutations)["notification"] = &graphql.Field{
		Type: mutation,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return mutation, nil
		},
	}
}

func NotificationSubscriptionMutationLeadsResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &notification.ListSubscribeLeads{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user != nil {
		if request.Email == "" {
			request.Email = user.GetAccount().GetContactValue().GetEmailPrimaryValue()
		}

		if request.Phone == "" {
			request.Phone = user.GetAccount().GetContactValue().GetPhonePrimaryValue()
		}

		if request.Name == "" {
			request.Name = fmt.Sprintf(
				"%s %s",
				user.GetAccount().GetDetailsValue().GetFirstNameValue(),
				user.GetAccount().GetDetailsValue().GetLastNameValue(),
			)
		}
	}

	_, err := connections.NotificationService.ListSubscribe(params.Context, &notification.ListSubscribeRequest{
		Subscription: &notification.ListSubscribeRequest_Leads{
			Leads: request,
		},
	})

	if err != nil {
		return false, err
	}

	return true, nil
}
